pub struct Cell<T> {
pub item: T,
pub height: u8,
pub left: Bst<T>,
pub right: Bst<T>,
}Expand description
Cell with item, left/right subtrees: look at the source code to understand. Note: usually items inside structs are not made public but I’ve chosen to do so to make your assignment easier.
Fields
item: Theight: u8left: Bst<T>right: Bst<T>Implementations